@@ -298,3 +298,12 @@ impl Bn {
         }
     }
 }
+
+#[test]
+fn safe_prime() {
+    let n = Bn::safe_prime(1024);
+    assert_eq!(n.0.bits(), 1024);
+    assert!(n.is_prime());
+    let sg: Bn = n >> 1;
+    assert!(sg.is_prime())
+}
